@@ -23,10 +23,10 @@ public class NearHundred extends BatExercise {
 		myWorld.addTest(INVISIBLE, 211);
 		myWorld.addTest(INVISIBLE, -100);
 
-		templatePython("nearHundred", 
+		templatePython("nearHundred", new String[]{"Int"},
 				"def nearHundred(n):\n",
 				"   return (90<=n and n<=110) or (190<=n and n<=210)\n");
-		templateScala("nearHundred",new String[]{"Int"},
+		templateScala("nearHundred", new String[]{"Int"},
 				"def nearHundred(n:Int): Boolean = {\n",
 				"  return (90<=n && n<=110)||(190<=n&&n<=210);\n"
 			  + "}");
