@@ -1,7 +1,7 @@
 colors = [Color.white, Color(255,240,240), Color(255,220,220), Color(255,205,205),
              Color(255,190,190), Color(255,170,170), Color(255,150,150),
              Color(255,130,130), Color(255,110,110), Color(255,45,45),
-             Color(255,5,5)]
+             Color(255,5,5),Color.magenta]
 
 def forward(i=-1):
     if i==-1:
@@ -9,7 +9,10 @@ def forward(i=-1):
       c = getGroundColor()
       for i in range(len(colors)-1):
           if colors[i] == c:
-              c = colors[i+1]
+              if i == len(colors)-1:
+                  c = colors[i]
+              else:
+                  c = colors[i+1]
               break
       setBrushColor(c)    
       brushDown()
@@ -18,7 +21,7 @@ def forward(i=-1):
       errorMsg("Sorry Dave, I cannot let you use forward with an argument in this exercise. Use a loop instead.")
 def backward(i=-1):
     if i==-1:
-      entity.backward()
+      errorMsg("Sorry Dave, you cannot run backward that way. Exercising is hard enough -- please don't overplay.")
     else:
       errorMsg("Sorry Dave, I cannot let you use backward with an argument in this exercise. Use a loop instead.")
 # BEGIN SOLUTION
