@@ -5,8 +5,8 @@ import java.awt.Color;
 import plm.core.model.lesson.ExerciseTemplated;
 import plm.core.model.lesson.Lesson;
 import plm.universe.Direction;
-import plm.universe.bugglequest.Buggle;
 import plm.universe.bugglequest.BuggleWorld;
+import plm.universe.bugglequest.SimpleBuggle;
 
 public class LoopWhile extends ExerciseTemplated {
 
@@ -16,7 +16,7 @@ public class LoopWhile extends ExerciseTemplated {
 
 		BuggleWorld myWorld = new BuggleWorld("Closed world",7,7);
 		for (int i=0;i<7;i++) {
-			new Buggle(myWorld, "Joker "+(i+1), i, 6, Direction.NORTH, Color.black, Color.lightGray);
+			new SimpleBuggle(myWorld, "Joker "+(i+1), i, 6, Direction.NORTH, Color.black, Color.lightGray);
 			myWorld.putTopWall (i, 6-i);
 			myWorld.putLeftWall(i, 6-i);
 			myWorld.putLeftWall(0, i  );
