@@ -6,6 +6,7 @@ import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.PrintStream;
 import java.lang.reflect.Method;
 import java.net.URL;
 import java.net.URLClassLoader;
@@ -27,29 +28,38 @@ import javax.script.ScriptEngineFactory;
 import javax.script.ScriptEngineManager;
 import javax.swing.JOptionPane;
 
+import org.eclipse.jgit.api.errors.GitAPIException;
 import org.xnap.commons.i18n.I18n;
 import org.xnap.commons.i18n.I18nFactory;
 
 import plm.core.GameListener;
 import plm.core.GameStateListener;
 import plm.core.HumanLangChangesListener;
+import plm.core.PLMEntityNotFound;
 import plm.core.ProgLangChangesListener;
 import plm.core.StatusStateListener;
+import plm.core.lang.LangC;
+import plm.core.lang.LangJava;
+import plm.core.lang.LangLightbot;
+import plm.core.lang.LangPython;
+import plm.core.lang.LangRuby;
+import plm.core.lang.LangScala;
+import plm.core.lang.ProgrammingLanguage;
 import plm.core.model.lesson.Exercise;
 import plm.core.model.lesson.Exercise.WorldKind;
 import plm.core.model.lesson.ExerciseTemplated;
 import plm.core.model.lesson.Lecture;
 import plm.core.model.lesson.Lesson;
+import plm.core.model.lesson.Lesson.LoadingOutcome;
+import plm.core.model.session.GitSessionKit;
 import plm.core.model.session.ISessionKit;
 import plm.core.model.session.SessionDB;
-import plm.core.model.session.ZipSessionKit;
+import plm.core.model.tracking.GitSpy;
 import plm.core.model.tracking.HeartBeatSpy;
 import plm.core.model.tracking.LocalFileSpy;
 import plm.core.model.tracking.ProgressSpyListener;
 import plm.core.model.tracking.ServerSpyAppEngine;
-import plm.core.model.tracking.TwitterSpy;
 import plm.core.ui.MainFrame;
-import plm.core.ui.ResourcesCache;
 import plm.core.utils.FileUtils;
 import plm.universe.Entity;
 import plm.universe.IWorldView;
@@ -83,31 +93,31 @@ public class Game implements IWorldView {
 	private Map<String, Lesson> lessons = new HashMap<String, Lesson>();
 	private Lesson currentLesson;
 	private Course currentCourse;
+	private Lecture lastExercise;
 
-	public static final String [][] humanLangs = { {"Francais","fr"}, {"English","en"}};
-	
-	public static final ProgrammingLanguage JAVA =       new ProgrammingLanguage("Java","java",ResourcesCache.getIcon("img/lang_java.png"));
-	public static final ProgrammingLanguage PYTHON =     new ProgrammingLanguage("Python","py",ResourcesCache.getIcon("img/lang_python.png"));
-	public static final ProgrammingLanguage SCALA =      new ProgrammingLanguage("Scala","scala",ResourcesCache.getIcon("img/lang_scala.png"));
+	public static final String [][] humanLangs = { {"English","en"}, {"Francais","fr"}, {"Italiano","it"}, {"Português brasileiro", "pt_BR"} };
+
+	public static final ProgrammingLanguage JAVA =       new LangJava();
+	public static final ProgrammingLanguage PYTHON =     new LangPython();
+	public static final ProgrammingLanguage SCALA =      new LangScala();
+	public static final ProgrammingLanguage C =          new LangC();
 	//public static final ProgrammingLanguage JAVASCRIPT = new ProgrammingLanguage("JavaScript","js",ResourcesCache.getIcon("img/lang_javascript.png"));
-	public static final ProgrammingLanguage RUBY =       new ProgrammingLanguage("Ruby","rb",ResourcesCache.getIcon("img/lang_ruby.png"));
-	public static final ProgrammingLanguage LIGHTBOT =   new ProgrammingLanguage("lightbot","ignored",ResourcesCache.getIcon("img/lightbot_light.png"));
+	public static final ProgrammingLanguage RUBY =       new LangRuby();
+	public static final ProgrammingLanguage LIGHTBOT =   new LangLightbot();
+	
 	public static final ProgrammingLanguage[] programmingLanguages = new ProgrammingLanguage[] {
-		JAVA, PYTHON, SCALA, RUBY, LIGHTBOT // TODO: re-add JAVASCRIPT to this list once it works at least a bit
+		JAVA, PYTHON, SCALA, RUBY, LIGHTBOT, C // TODO: re-add JAVASCRIPT to this list once it works at least a bit
 	}; 
 	private ProgrammingLanguage programmingLanguage = JAVA;
 
 	/* TODO: document these values elsewhere */
 	public static final String PROP_OUTPUT_CAPTURE = "output.capture"; // Whether to redirect stdout and stderr to the graphical console. Defaults to true
 	public static final String PROP_ANSWER_CACHE = "answers.cache"; // Whether to use the cache of answers worlds on disk, defaults to true. 
-	                                                                // Turning to false will slow down the startup process, but avoid out of date files
-	
-	public static final String PROP_PROGRESS_TWITTER = "plm.progress.twitter";     //  
+	// Turning to false will slow down the startup process, but avoid out of date files
+
 	public static final String PROP_PROGRESS_APPENGINE = "plm.progress.appengine"; // Whether the progresses should be posted to the appengine (default: false)
 	public static final String PROP_APPENGINE_URL = "plm.appengine.url"; // Where to find the appengine. This is related to the teacher console, that should be rewritten at some point.
-	
-	public static final String SESSION_CLOUD_PROVIDER_URL = "plm.session.cloud.provider.url";
-	
+
 	public static final String PROP_PROGRAMING_LANGUAGE = "plm.programingLanguage";
 
 	public static final String PROP_FONT_SIZE = "plm.display.fontsize"; // the CSS property of the font size
@@ -120,23 +130,28 @@ public class Game implements IWorldView {
 	private List<Thread> demoRunners = new ArrayList<Thread>();
 	private static List<Thread> initRunners = new ArrayList<Thread>();
 
-    private HeartBeatSpy heartBeatSpy;
+	private HeartBeatSpy heartBeatSpy;
 
 	private ArrayList<GameStateListener> gameStateListeners = new ArrayList<GameStateListener>();
 
 	private LogWriter outputWriter;
+	private PrintStream outputOrig = System.out;
+	private PrintStream errorOrig = System.err;
+
 
 	public SessionDB studentWork = new SessionDB();
-	private ISessionKit sessionKit = new ZipSessionKit(this);
+	//private ISessionKit sessionKit = new ZipSessionKit(this);
+	private ISessionKit sessionKit;
+
+	private Users users;
 
 	private static boolean ongoingInitialization = false;
-	private static String lessonChooser = "lessons.chooser";
 	public  static I18n i18n;
 
 	public static Game getInstance() {
 		if (Game.instance == null) {
 			if (ongoingInitialization)
-				throw new RuntimeException("Loop in initialization process. This is a PLM bug.");
+				throw new RuntimeException("Loop in initialization process. This is a PLM bug: please report it.");
 			ongoingInitialization = true;
 			Game.instance = new Game();
 			ongoingInitialization = false;
@@ -152,27 +167,35 @@ public class Game implements IWorldView {
 		if (checkScala())
 			System.err.println(i18n.tr("Scala is usable on your machine. Congratulations."));
 		else
-			System.err.println(i18n.tr("Please install Scala version 2.10 or higher to use it in PLM."));
+			System.err.println(i18n.tr("Please install Scala version 2.10 or higher to use it in the PLM."));
 		if (checkPython())
 			System.err.println(i18n.tr("Jython is usable on your machine. Congratulations."));
 		else
-			System.err.println(i18n.tr("Please install jython to use the python programming language in PLM."));
-		
-		String defaultProgrammingLanguage = Game.getProperty(PROP_PROGRAMING_LANGUAGE,"Java",true);
+			System.err.println(i18n.tr("Please install jython to use the python programming language in the PLM."));
+		if (checkC())
+			System.err.println(i18n.tr("C is usable on your machine. Congratulations."));
+		else
+			System.err.println(i18n.tr("Please install gcc to use the C programming language in the PLM."));
+
+		String defaultProgrammingLanguage = Game.getProperty(PROP_PROGRAMING_LANGUAGE,Game.JAVA.getLang(),true);
 		if (!defaultProgrammingLanguage.equalsIgnoreCase(Game.JAVA.getLang()) &&
-			!defaultProgrammingLanguage.equalsIgnoreCase(Game.PYTHON.getLang()) &&
-			!defaultProgrammingLanguage.equalsIgnoreCase(Game.SCALA.getLang())) 
-			System.err.println(i18n.tr("Warning, the default programming language is neither ''Java'' nor ''python'' or ''Scala'' but {0}.\n"+
+				!defaultProgrammingLanguage.equalsIgnoreCase(Game.PYTHON.getLang()) &&
+				!defaultProgrammingLanguage.equalsIgnoreCase(Game.SCALA.getLang()) && 
+				!defaultProgrammingLanguage.equalsIgnoreCase(Game.C.getLang())) 
+			System.err.println(i18n.tr("Warning, the default programming language is neither ''Java'' nor ''python'' or ''Scala'' or ''C'' but {0}.\n"+
 					"   This language will be used to setup the worlds, possibly leading to severe issues for the exercises that don''t expect it.\n" +
-					"   It is safer to change the current language, and restart PLM before proceeding.\n"+
-					"   Alternatively, the property {1} can be changed in your configuration file ($HOME/.plm/plm.properties)",defaultProgrammingLanguage,PROP_PROGRAMING_LANGUAGE));
-		
+					"   It is safer to change the current language, and restart the PLM before proceeding.\n"+
+					"   Alternatively, the property {1} can be changed in your configuration file ({2}/plm.properties)",defaultProgrammingLanguage,PROP_PROGRAMING_LANGUAGE, getSavingLocation()));
+
 		if (defaultProgrammingLanguage.equalsIgnoreCase(Game.SCALA.getLang()) && !canScala) {
 			System.err.println(i18n.tr("The default programming language is Scala, but your scala installation is not usable. Switching to Java instead.\n"));
 			setProgramingLanguage(JAVA);
 		} else if (defaultProgrammingLanguage.equalsIgnoreCase(Game.PYTHON.getLang()) && !canPython) {
 			System.err.println(i18n.tr("The default programming language is python, but your python installation is not usable. Switching to Java instead.\n"));
 			setProgramingLanguage(JAVA);
+		} else if (defaultProgrammingLanguage.equalsIgnoreCase(Game.C.getLang()) && !canC) {
+			System.err.println(i18n.tr("The default programming language is C, but your C installation is not usable. Switching to Java instead.\n"));
+			setProgramingLanguage(JAVA);
 		} else {
 			for (ProgrammingLanguage pl : Game.getProgrammingLanguages()) 
 				if (pl.getLang().equals(defaultProgrammingLanguage)) {
@@ -180,24 +203,32 @@ public class Game implements IWorldView {
 					break;
 				}
 		}
-				
+
+		users = new Users(SAVE_DIR);
+		users.getCurrentUser();
+
 		addProgressSpyListener(new LocalFileSpy(SAVE_DIR));
-		
-		if (getProperty(PROP_PROGRESS_TWITTER, "true",true).equalsIgnoreCase("true")) {
-			System.err.println(i18n.tr("Your progress will be posted to https://twitter.com/jlmlovers This can be turned off through the property {0}",Game.PROP_PROGRESS_TWITTER));
-			addProgressSpyListener(new TwitterSpy());
-		} else {
-			System.err.println(i18n.tr("Your progress will NOT be posted to twitter, as requested by the property {0}",Game.PROP_PROGRESS_TWITTER));			
+		sessionKit = new GitSessionKit(this);
+
+		try {
+			addProgressSpyListener(new GitSpy(SAVE_DIR, users));
+		} catch (IOException | GitAPIException e) {
+			System.err.println(Game.i18n.tr("You found a bug in the PLM. Please report it with all possible details (including the stacktrace below"));
+			e.printStackTrace();
 		}
+
+
 		if (getProperty(PROP_PROGRESS_APPENGINE, "false",true).equalsIgnoreCase("true"))
 			addProgressSpyListener(new ServerSpyAppEngine());
-
-        currentCourse = new CourseAppEngine();
+		
+		if (! Game.getProperty(Game.PROP_APPENGINE_URL).equals("")) { // FIXME: there is no way real proper way to disable the CourseEngine !!!
+	        currentCourse = new CourseAppEngine();
+		}
 	}
-	
+
 	boolean canScala = false;
 	String scalaError = "";
-	
+
 	@SuppressWarnings({ "rawtypes", "unchecked" })
 	private boolean checkScala() {
 		String[] resources = new String[] {"/scala/tools/nsc/Interpreter", "/scala/ScalaObject", "/scala/reflect/io/AbstractFile"};
@@ -230,16 +261,16 @@ public class Game implements IWorldView {
 			return canScala;
 		}
 	}
-	
+
 	public boolean canPython = false;
 	String pythonError = "";
 	private boolean checkPython() {
 		String[] resources = new String[] {
 				"/org/python/jsr223/PyScriptEngineFactory", "/org/jruby/ext/posix/util/Platform","/org/antlr/runtime/CharStream",
 				"/org/objectweb/asm/Opcodes"
-				};
+		};
 		String[] hints     = new String[] {"jython.jar", "jruby.jar","antlr3-runtime.jar",
-				"asm3.jar"};
+		"asm3.jar"};
 		for (int i=0;i<resources.length;i++) {
 			pythonError = canResolve(resources[i],hints[i]);
 			if (!pythonError.isEmpty()) {
@@ -247,7 +278,7 @@ public class Game implements IWorldView {
 				return canPython;
 			}
 		}
-		
+
 		ScriptEngineManager manager = new ScriptEngineManager();       
 		if (manager.getEngineByName("python") == null) {
 			pythonError = i18n.tr("Cannot retrieve the python ScriptEngine. Are jython.jar and its dependencies in the classpath?");
@@ -257,37 +288,44 @@ public class Game implements IWorldView {
 		return true;
 	}
 
+
+	public boolean canC = false;
+	String CError = "";
+	private boolean checkC(){
+		Runtime runtime = Runtime.getRuntime();
+		try {
+			runtime.exec("gcc --version");
+			canC=true;
+		} catch (IOException e) {
+			e.printStackTrace();
+			canC=false;
+		}
+		return canC;
+	}
+
+
 	private String canResolve(String resource, String hint) {
 		try {
 			URL path = getClass().getResource(resource+".class");
 			if (path != null)
 				return ""; // Cool, found it.
-				
+
 			path = ClassLoader.getSystemResource(resource+".class");
 			if (path != null)
 				return ""; // Cool, found it.
-			
+
 			resource = resource.replaceAll("/", ".");
 			resource = resource.substring(1);
 			Class.forName(resource).newInstance();
 			return ""; // That's cool if I manage to create one such object
-			
+
 		} catch (ClassNotFoundException ce) {
 			return i18n.tr("Resource {0} not found in the classpath.\nIs {1} in your classpath?",resource,hint);
 		} catch (Exception e) {
 			return i18n.tr("{0} received while searching for resource {1}: {2}",e.getClass().getName(),resource,e.getLocalizedMessage());
 		}
 	}
-		
-	
-	/**
-	 * Load the chooser, stored in plm.core.ui.chooser
-	 */
-	public void loadChooser() {
-		Game.instance.switchLesson(lessonChooser,false);
-	}
-	
-	
+
 	/** Change the current lesson.
 	 * 
 	 * Also, initialize the newly used lesson on need. It must already be in the classpath 
@@ -295,23 +333,23 @@ public class Game implements IWorldView {
 	 *  
 	 * @param lessonName package name of the lesson to load 
 	 */
-	
+
 	public Lesson switchLesson(String lessonName, boolean failOnError) {
 		if (stepModeEnabled())
 			disableStepMode();
-		
+
 		this.setState(GameState.LOADING);
 		// Try caching the lesson to avoid the possibly long loading time during which we compute the solution of each exercise  
 		Lesson lesson = lessons.get(lessonName);
 		statusArgAdd(lessonName);
-		
+
 		if (lesson == null) { // we have to load it 
 			try {
 				// This is where we assume here that each lesson contains a Main object, instantiating the Lesson class.
 				// We manually build a call to the constructor of this object, and fire it
 				// This creates such an object, which is in charge of creating the whole lesson (including exercises) from its constructor
 				lesson = (Lesson) Class.forName(lessonName + ".Main").newInstance();
-				
+
 				lessons.put(lessonName, lesson); // cache the newly created object
 			} catch (InstantiationException e) {
 				e.printStackTrace();
@@ -325,8 +363,8 @@ public class Game implements IWorldView {
 				return getCurrentLesson();
 			}
 		}
-		// Prevent an error message telling us that the PLM couldn't load our code for the chooser -- kinda obvious
-		if ( !lessonName.equals(lessonChooser) && sessionKit != null)
+		// Prevent obvious error messages
+		if (sessionKit != null)
 			sessionKit.loadLesson(SAVE_DIR, lesson);
 		try {
 			waitInitThreads();
@@ -334,10 +372,16 @@ public class Game implements IWorldView {
 			System.err.println("Interrupted while loading the lesson "+lesson.getName());
 			e.printStackTrace();
 		}
+		// If a problem arose while setting up the lesson, don't switch 
+		// TODO: define a better solution when encountering this issue
+		if(lesson.getLoadingOutcomeState() == LoadingOutcome.FAIL) {
+			JOptionPane.showMessageDialog(null, i18n.tr("The lesson "+lesson.getName()+" encountered an issue while loading its exercises, please choose another lesson.") ,
+					i18n.tr("Broken lesson"), JOptionPane.ERROR_MESSAGE); 
+			return null;
+		}
+			
 		setCurrentLesson(lesson);
-		
 		this.setState(GameState.LOADING_DONE);
-
 		return lesson;
 	}
 	private Set<String> usedJARs = new HashSet<String>(); // cache used in loadLessonFromJAR()
@@ -350,37 +394,37 @@ public class Game implements IWorldView {
 	 * @throws LessonLoadingException if the JAR file is inexistent or invalid (or if the system classloader does not accept URLs)
 	 */
 	public void loadLessonFromJAR(File jar) throws LessonLoadingException {
-		
+
 		if (!jar.exists())
 			throw new LessonLoadingException("File "+jar.getName()+" does not exist");
-		
-		
+
+
 		// Check if the JAR has already been added. If not, load it in the classloader.
 		if (!usedJARs.contains(jar.getAbsolutePath())) {	
 			URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
 			Class<URLClassLoader> sysclass = URLClassLoader.class;
-			
+
 			URL urlOfJar;
 			try {
 				urlOfJar = jar.toURI().toURL();
 			} catch (IOException e1) {
 				throw new LessonLoadingException("Error while reading the jarfile "+jar.getName(),e1);
 			}
-	        
-	        try {
-	        	// Hell yeah. That method is usually private, but I can change it that easily.
-	        	// Some people even call this bullshit "security"...
-	        	
-	            Method method = sysclass.getDeclaredMethod("addURL",new Class[]{URL.class});
-	            method.setAccessible(true);
-	            method.invoke(sysloader,new Object[]{ urlOfJar });
-	        } catch (Throwable t) {
-	        	throw new LessonLoadingException("Internal Error: The system classloader refused to load the URL of this lesson file. You may want to change the JVM classloader from the command line.",t);
-	        }
-	        
-	        usedJARs.add(jar.getAbsolutePath());
+
+			try {
+				// Hell yeah. That method is usually private, but I can change it that easily.
+				// Some people even call this bullshit "security"...
+
+				Method method = sysclass.getDeclaredMethod("addURL",new Class[]{URL.class});
+				method.setAccessible(true);
+				method.invoke(sysloader,new Object[]{ urlOfJar });
+			} catch (Throwable t) {
+				throw new LessonLoadingException("Internal Error: The system classloader refused to load the URL of this lesson file. You may want to change the JVM classloader from the command line.",t);
+			}
+
+			usedJARs.add(jar.getAbsolutePath());
 		}
-        
+
 		// Load the JAR manifest file to retrieve the lesson's package name in it
 		Manifest manifest;
 		try {
@@ -388,16 +432,16 @@ public class Game implements IWorldView {
 		} catch (Exception e) {
 			throw new LessonLoadingException("Invalid lesson file (Manifest not found): "+jar.getName(), e);
 		}
-		
+
 		String lessonPackage = manifest.getMainAttributes().getValue("LessonPackage");
 		if (lessonPackage == null)
 			throw new LessonLoadingException("Invalid lesson file (Attribute 'LessonPackage' not found in Manifest): "+jar.getName());
-		
+
 		// We are ready to launch this lesson
 		Game.getInstance().switchLesson("lessons." + lessonPackage,false);
-    }//end method
+	}//end method
+
 
-	
 
 	public static void addInitThread(Thread t) {
 		initRunners.add(t);
@@ -419,23 +463,27 @@ public class Game implements IWorldView {
 	}
 
 	public void setCurrentLesson(Lesson lesson) {
-		try {
-			saveSession(); // don't loose user changes 
-			
-			this.currentLesson = lesson;
-			fireCurrentLessonChanged();
-			setCurrentExercise(this.currentLesson.getCurrentExercise());
-			
-		} catch (UserAbortException e) { 
-			System.out.println(i18n.tr("Operation cancelled by the user"));
-		}
+		// Don't set the currentLesson from here, or the lastExercise saving mechanism will not work
+		setCurrentExercise(lesson.getCurrentExercise());
+	}
+
+	public Lecture getLastExercise() {
+		return this.lastExercise;
 	}
 
 	// only to avoid that exercise views register as listener of a lesson
 	public void setCurrentExercise(Lecture lect) {
 		try {
-			saveSession(); // don't loose user changes 
-
+			saveSession(); // don't loose user changes
+			this.lastExercise = (currentLesson==null ? null : currentLesson.getCurrentExercise()); // save the last viewed exercise before switching7
+			
+			if (this.currentLesson != lect.getLesson())
+				this.currentLesson = lect.getLesson();
+			
+			/* if the user changes the exercise, you can assume that he wants to test another challenge */
+			if (isCreativeEnabled())
+				switchCreative();
+			
 			this.currentLesson.setCurrentExercise(lect);
 			fireCurrentExerciseChanged(lect);
 			if (lect instanceof Exercise) {
@@ -451,21 +499,23 @@ public class Game implements IWorldView {
 						fallback = l;
 				}
 				/* Use the first (programming) language advertised by the exercise java as a fallback */
-				System.out.println( 
-						Game.i18n.tr("Exercise {0} does not support language {1}. Fallback to {2} instead. "
-								+ "Please consider contributing to this project by adapting this exercise to this language.",
-								lect.getName(),getProgrammingLanguage(),fallback.getLang()));
+				if (getProgrammingLanguage() != Game.LIGHTBOT && fallback != Game.LIGHTBOT)
+					System.out.println(
+							Game.i18n.tr("Exercise {0} does not support language {1}. Fallback to {2} instead. "
+									+ "Please consider contributing to this project by adapting this exercise to this language.",
+									lect.getName(),getProgrammingLanguage(),fallback.getLang()));
 				setProgramingLanguage(fallback);
-				
+
 			}
 			MainFrame.getInstance().currentExerciseHasChanged(lect); // make sure that the right language is selected -- yeah that's a ugly way of doing it
-
 		} catch (UserAbortException e) { 
 			System.out.println(i18n.tr("Operation cancelled by the user"));
 		}
 	}
 
 	public World getSelectedWorld() {
+		if (this.currentLesson == null)
+			return null;
 		Lecture lecture = this.currentLesson.getCurrentExercise();
 		if (lecture instanceof Exercise) {
 			if (this.selectedWorld == null) {
@@ -531,7 +581,7 @@ public class Game implements IWorldView {
 
 		if (runner != null)
 			runner.stopAll();
-		
+
 		Lecture lecture = this.currentLesson.getCurrentExercise();
 		if (lecture instanceof Exercise)
 			for (World w : ((Exercise) lecture).getWorlds(WorldKind.ANSWER))
@@ -585,14 +635,19 @@ public class Game implements IWorldView {
 		return this.state;
 	}
 
-	public void setOutputWriter(LogWriter writer) {
-		this.outputWriter = writer;
-		if (getProperty(PROP_OUTPUT_CAPTURE, "true",true).equals("true")) {
+	public void setCaptureOutput(boolean isCaptured) {
+		if (isCaptured && getProperty(PROP_OUTPUT_CAPTURE, "true",true).equals("true")) {
 			Logger l = new Logger(outputWriter);
 			System.setOut(l);
 			System.setErr(l);
+		} else if (System.out.equals(this.outputOrig)) {
+			System.setOut(this.outputOrig);
+			System.setErr(this.errorOrig);
 		}
 	}
+	public void setOutputWriter(LogWriter writer) {
+		this.outputWriter = writer;
+	}
 
 	public LogWriter getOutputWriter() {
 		return this.outputWriter;
@@ -602,13 +657,13 @@ public class Game implements IWorldView {
 		try {
 			// FIXME: this method is not called when pressing APPLE+Q on OSX
 
-            // report user leave on the server
-            for(ProgressSpyListener spyListener: progressSpyListeners){
-                spyListener.leave();
-            }
-            // stop the heartbeat report to PLMServer
-            if(heartBeatSpy != null)
-                heartBeatSpy.die();
+			// report user leave on the server
+			for(ProgressSpyListener spyListener: progressSpyListeners){
+				spyListener.leave();
+			}
+			// stop the heartbeat report to PLMServer
+			if(heartBeatSpy != null)
+				heartBeatSpy.die();
 
 			saveSession();
 			storeProperties();
@@ -656,7 +711,7 @@ public class Game implements IWorldView {
 		sessionKit = null;
 		System.out.println("Disabling the session kit on disk.");
 	}
-	
+
 	public static void loadProperties() {
 		InputStream is = null;
 		try {
@@ -747,26 +802,20 @@ public class Game implements IWorldView {
 		this.listeners.remove(l);
 	}
 
-	protected void fireCurrentLessonChanged() {
-		for (GameListener v : this.listeners) {
-			v.currentLessonHasChanged();
-		}
-	}
-
-    protected void fireCurrentExerciseChanged(Lecture lect) {
+	protected void fireCurrentExerciseChanged(Lecture lect) {
 		if (stepModeEnabled())
 			disableStepMode();
 
-        for (GameListener v : this.listeners) {
-            v.currentExerciseHasChanged(lect);
-        }
+		for (GameListener v : this.listeners) {
+			v.currentExerciseHasChanged(lect);
+		}
 
-        if(lect instanceof Exercise){
-            for(ProgressSpyListener p: this.progressSpyListeners){
-                p.switched((Exercise)lect);
-            }
-        }
-    }
+		if(lect instanceof Exercise){
+			for(ProgressSpyListener p: this.progressSpyListeners){
+				p.switched((Exercise)lect);
+			}
+		}
+	}
 
 	protected void fireSelectedWorldHasChanged(World w) {
 		for (GameListener v : this.listeners) {
@@ -811,7 +860,25 @@ public class Game implements IWorldView {
 			l.executed(exo);
 		}
 	}
+	
+	public void fireCallForHelpSpy(String studentInput) {
+		for (ProgressSpyListener l : this.progressSpyListeners) {
+			l.callForHelp(studentInput);
+		}
+	}
+	
+	public void fireCancelCallForHelpSpy() {
+		for (ProgressSpyListener l : this.progressSpyListeners) {
+			l.cancelCallForHelp();
+		}
+	}
 
+	public void fireReadTipSpy(String id, String mission) {
+		for (ProgressSpyListener l : this.progressSpyListeners) {
+			l.readTip(id, mission);
+		}
+	}
+	
 	@Override
 	public void worldHasChanged() {
 		if (selectedWorld.getEntityCount()>0)
@@ -865,7 +932,7 @@ public class Game implements IWorldView {
 				sb.append(", ");
 			sb.append(s);
 		}
-		
+
 		String msg = first ? "" : sb.toString(); // remove everything if no argument at all 
 		for (StatusStateListener l : this.statusStateListeners) 
 			l.stateChanged(msg);
@@ -906,19 +973,34 @@ public class Game implements IWorldView {
 		if (isValidProgLanguage(newLanguage)) {
 			//System.out.println("Switch programming language to "+newLanguage);
 			if (newLanguage.equals(Game.SCALA) && !canScala) {
-				JOptionPane.showMessageDialog(null, i18n.tr("Please install Scala version 2.10 or higher to use it in PLM.\n\n")+scalaError ,
+				JOptionPane.showMessageDialog(null, i18n.tr("Please install Scala version 2.10 or higher to use it in the PLM.\n\n")+scalaError ,
 						i18n.tr("Scala is missing"), JOptionPane.ERROR_MESSAGE); 
 				return;
 			}
 			if (newLanguage.equals(Game.PYTHON) && !canPython) {
-				JOptionPane.showMessageDialog(null, i18n.tr("Please install jython and its dependencies to use the python programming language in PLM.\n\n")+pythonError,
+				JOptionPane.showMessageDialog(null, i18n.tr("Please install jython and its dependencies to use the python programming language in the PLM.\n\n")+pythonError,
 						i18n.tr("Python is missing"), JOptionPane.ERROR_MESSAGE); 
 				return;
 			}
+			if (newLanguage.equals(Game.C) && !canC) {
+				JOptionPane.showMessageDialog(null, i18n.tr("Please install C and its dependencies to use the C programming language in the PLM.\n\n")+CError,
+						i18n.tr("C is missing"), JOptionPane.ERROR_MESSAGE); 
+				return;
+			}
+			if (newLanguage.equals(Game.C) && !doBatch) {
+				int res = JOptionPane.showConfirmDialog(null, 
+						i18n.tr(  "The C langage is currently very experimental in the PLM.\n"
+			                    + "If you go for C, you may not be able to complete some exercises that\n"
+			                    + "are still in progress in C, although some other parts are already okay.\n\n"
+			                    + "Do you want to proceed anyway?"),
+						i18n.tr("C is still experimental"), JOptionPane.OK_CANCEL_OPTION);
+				if (res != JOptionPane.OK_OPTION)
+					return;
+			}
 			this.programmingLanguage = newLanguage;
 			fireProgLangChange(newLanguage);
-			if (newLanguage.equals(Game.JAVA) || newLanguage.equals(Game.PYTHON) || newLanguage.equals(Game.SCALA)) // Only save it if it's stable enough
-				setProperty(PROP_PROGRAMING_LANGUAGE, newLanguage.lang);
+			if (newLanguage.equals(Game.JAVA) || newLanguage.equals(Game.PYTHON) || newLanguage.equals(Game.SCALA) || newLanguage.equals(Game.C)) // Only save it if it's stable enough
+				setProperty(PROP_PROGRAMING_LANGUAGE, newLanguage.getLang());
 			return;
 		}
 		throw new RuntimeException("Ignoring request to switch the programming language to the unknown "+newLanguage);
@@ -963,7 +1045,7 @@ public class Game implements IWorldView {
 	public void removeHumanLangListener(HumanLangChangesListener l) {
 		this.humanLangListeners.remove(l);
 	}
-	
+
 	private boolean doDebug = false;
 	public void switchDebug() {
 		doDebug = !doDebug;
@@ -972,28 +1054,40 @@ public class Game implements IWorldView {
 			System.out.println("Saving location: "+SAVE_DIR.getAbsolutePath());
 			System.out.println("Lesson: "+(l==null?"None loaded yet":l.getName()));
 			System.out.println("Exercise: "+(l==null?"None loaded yet":l.getCurrentExercise().getName()));
+			for (World w:((Exercise)l.getCurrentExercise()).getWorlds(WorldKind.ANSWER)) {
+				String s = w.getDebugInfo();
+				if (s != "") 
+					System.out.println("World: "+s);
+			}
 			System.out.println("PLM version: "+Game.getProperty("plm.major.version","internal",false)+" ("+Game.getProperty("plm.major.version","internal",false)+"."+Game.getProperty("plm.minor.version","",false)+")");
 			System.out.println("Java version: "+System.getProperty("java.version")+" (VM: "+ System.getProperty("java.vm.name")+" "+ System.getProperty("java.vm.version")+")");
 			System.out.println("System: " +System.getProperty("os.name")+" (version: "+System.getProperty("os.version")+"; arch: "+ System.getProperty("os.arch")+")");
 			for (ScriptEngineFactory sef : new ScriptEngineManager().getEngineFactories()) {
-				  System.out.println(sef);
-				  System.out.append("  Engine: ")
-				      .append(sef.getEngineName())
-				      .append(" ")
-				      .println(sef.getEngineVersion());
-				  System.out.append("  Language: ")
-				      .append(sef.getLanguageName())
-				      .append(" ")
-				      .println(sef.getLanguageVersion());
-				  System.out.append("  Names: ")
-				      .println(sef.getNames());
-				}
+				System.out.println(sef);
+				System.out.append("  Engine: ")
+				.append(sef.getEngineName())
+				.append(" ")
+				.println(sef.getEngineVersion());
+				System.out.append("  Language: ")
+				.append(sef.getLanguageName())
+				.append(" ")
+				.println(sef.getLanguageVersion());
+				System.out.append("  Names: ")
+				.println(sef.getNames());
+			}
 		}
 	}
 	public boolean isDebugEnabled() {
 		return doDebug;
 	}
-
+	private boolean doBatch = false;
+	public void setBatchExecution() {
+		doBatch = true;
+	}
+	public boolean isBatchExecution() {
+		return doBatch;
+	}
+	
 	private boolean doCreative = false;		
 	public void switchCreative() {
 		doCreative =  !doCreative;
@@ -1003,47 +1097,54 @@ public class Game implements IWorldView {
 	}
 
 	/*
-     * Getter and Setter for the course ID for the current session.
-     * This ID will be used by the ServerSpy, to associate this
-     * PLM student with a course started by a teacher on the server
-     */
-    public String getCourseID() {
-    	if (this.currentCourse == null)
-    		return "";
-    	else
-    		return currentCourse.getCourseId();
-    }
+	 * Getter and Setter for the course ID for the current session.
+	 * This ID will be used by the ServerSpy, to associate this
+	 * PLM student with a course started by a teacher on the server
+	 */
+	public String getCourseID() {
+		if (this.currentCourse == null)
+			return "";
+		else
+			return currentCourse.getCourseId();
+	}
 
-    public String getCoursePassword(){
-        if(this.currentCourse == null)
-            return "";
-        else
-            return currentCourse.getPassword();
-    }
+	public String getCoursePassword(){
+		if(this.currentCourse == null)
+			return "";
+		else
+			return currentCourse.getPassword();
+	}
 
-    public void setCourseID(String courseID) {
-    	this.currentCourse.setCourseId(courseID);
-    }
+	public void setCourseID(String courseID) {
+		this.currentCourse.setCourseId(courseID);
+	}
 
-    public Course getCurrentCourse() {
-        return currentCourse;
-    }
+	public Course getCurrentCourse() {
+		return currentCourse;
+	}
 
-    public void setCurrentCourse(Course currentCourse) {
-        this.currentCourse = currentCourse;
-    }
+	public void setCurrentCourse(Course currentCourse) {
+		this.currentCourse = currentCourse;
+	}
 
-    public HeartBeatSpy getHeartBeatSpy(){ return this.heartBeatSpy; }
+	public Users getUsers() {
+		return users;
+	}
 
-    public void setHeartBeatSpy(HeartBeatSpy heartBeatSpy){ this.heartBeatSpy = heartBeatSpy; }
+	public void setUsers(Users users) {
+		this.users = users;
+	}
+
+	public HeartBeatSpy getHeartBeatSpy(){ return this.heartBeatSpy; }
+
+	public void setHeartBeatSpy(HeartBeatSpy heartBeatSpy){ this.heartBeatSpy = heartBeatSpy; }
+
+	public ArrayList<ProgressSpyListener> getProgressSpyListeners(){ return this.progressSpyListeners; }
+
+	/* Mechanism to find where to save our data */
+
+	private static String HOME_DIR = System.getProperty("user.home");
 
-    public ArrayList<ProgressSpyListener> getProgressSpyListeners(){ return this.progressSpyListeners; }
-    
-    
-    /* Mechanism to find where to save our data */
-	
-    private static String HOME_DIR = System.getProperty("user.home");
-	
 	/* These names are tested one after the other, searching for one that exist or that we can create */
 	static String[] rootDirNames = new String[] { 
 		HOME_DIR + File.separator + ".plm", /* preferred, default directory name */
