@@ -12,4 +12,9 @@ public interface ProgressSpyListener {
     public String join();
 
     public void leave();
+	
+	public void callForHelp(String studentInput);
+	public void cancelCallForHelp();
+
+	public void readTip(String id, String mission);
 }
