@@ -2,8 +2,12 @@ package plm.universe.bugglequest;
 
 import java.awt.Color;
 import java.awt.Point;
+import java.io.BufferedWriter;
+import java.io.IOException;
 
 import plm.core.model.Game;
+import plm.core.utils.ColorMapper;
+import plm.core.utils.InvalidColorNameException;
 import plm.universe.Direction;
 import plm.universe.Entity;
 import plm.universe.GridWorld;
@@ -17,20 +21,25 @@ import plm.universe.bugglequest.exception.NoBaggleUnderBuggleException;
 public abstract class AbstractBuggle extends Entity {
 	int k_val = 0;
 	int[] k_seq = {0,0, 1,1, 2,3, 2,3, 4,5};
-	
-	Color bodyColor;
-	
-	Color brushColor;
 
-	private int x;
-	private int y;
+	Color bodyColor = Color.red;
+	Color brushColor = Color.red;
+	
+	private boolean dontIgnoreDirectionDifference = true; // if the buggle direction matters for world equality
 
-	Direction direction;
+
+	private int x = 0;
+	private int y = 0;
+
+	Direction direction = Direction.NORTH;
 
 	boolean brushDown;
 
 	private boolean carryBaggle;
 
+	/* used to tell the observers what was changed */
+	public static final int BRUSH_STATE = 0, BRUSH_COLOR = 1, BUGGLE_COLOR = 2;
+
 	/* This is for the simple buggle to indicate that it did hit a wall, and is thus not a valid
 	 * candidate for exercise completion.
 	 */
@@ -45,25 +54,13 @@ public abstract class AbstractBuggle extends Entity {
 	public boolean haveSeenError() {
 		return seenError;
 	}	
-	
-	/**
-	 * Constructor with no argument so that child classes can avoid declaring a
-	 * constructor. But it should not be used as most methods assert on world
-	 * being not null. After using it, {@link Entity#setWorld(BuggleWorld)} must be used
-	 * ASAP.
-	 */
+
+	/** The PLM calls that constructor with no parameter, so it must exist (but you probably don't want to use it yourself). */
 	public AbstractBuggle() {
-		this(null, "John Doe", 0, 0, Direction.NORTH, Color.red, Color.red);
-	}
-
-	public AbstractBuggle(BuggleWorld w) {
-		this(w, "John Doe", 0, 0, Direction.NORTH, Color.red, Color.red);
-	}
-
-	public AbstractBuggle(BuggleWorld world, String name, int x, int y, Direction direction, Color c) {
-		this(world, name, 0, 0, Direction.NORTH, c, c);
+		super();
 	}
 
+	/** That constructor is called by the exercises */
 	public AbstractBuggle(World world, String name, int x, int y, Direction direction, Color color, Color brushColor) {
 		super(name,world);
 		this.bodyColor = color;
@@ -82,10 +79,6 @@ public abstract class AbstractBuggle extends Entity {
 		this.y = other.y;
 		this.direction = other.direction;
 	}
-	@Override
-	public Entity copy() {
-		return new Buggle(this);
-	}
 
 	public boolean isBrushDown() {
 		return brushDown;
@@ -96,11 +89,15 @@ public abstract class AbstractBuggle extends Entity {
 		BuggleWorldCell cell = (BuggleWorldCell) ((BuggleWorld)world).getCell(x, y);
 		cell.setColor(brushColor);
 		world.notifyWorldUpdatesListeners();
+		setChanged();
+		notifyObservers(BRUSH_STATE);
 	}
 
 	public void brushUp() {
 		if (k_seq[k_val]==4) k_val++; else k_val = 0;
 		this.brushDown = false;
+		setChanged();
+		notifyObservers(BRUSH_STATE);
 	}
 
 	public Color getGroundColor() {
@@ -116,6 +113,8 @@ public abstract class AbstractBuggle extends Entity {
 			brushColor = c;
 		if (brushDown) // mark the ground
 			brushDown();
+		setChanged();
+		notifyObservers(BRUSH_COLOR);
 	}
 
 	public Color getBodyColor() {
@@ -126,6 +125,8 @@ public abstract class AbstractBuggle extends Entity {
 		if (c != null) {
 			this.bodyColor = c;
 			world.notifyWorldUpdatesListeners();
+			setChanged();
+			notifyObservers(BUGGLE_COLOR);
 		}
 	}
 
@@ -149,15 +150,23 @@ public abstract class AbstractBuggle extends Entity {
 		if (k_seq[k_val]==3) k_val++; else k_val = 0;
 		setDirection(direction.right());
 	}
+	
+	// Make sure that the case issue is detected in Scala by overriding the Left() and Right() methods (see #236)
+	public void Left() { 
+		throw new RuntimeException(Game.i18n.tr("Sorry Dave, I cannot let you use Left() with an uppercase. Use left() instead."));
+	}
+	public void Right() {
+		throw new RuntimeException(Game.i18n.tr("Sorry Dave, I cannot let you use Right() with an uppercase. Use right() instead."));
+	}
 
 	public void back() {
 		setDirection(direction.opposite());
 	}
-	
+
 	public int getWorldHeight() {
 		return ((GridWorld) world).getHeight();
 	}
-	
+
 	public int getWorldWidth() {
 		return ((GridWorld) world).getWidth();
 	}
@@ -233,7 +242,7 @@ public abstract class AbstractBuggle extends Entity {
 		try {
 			setPos(x,y);
 		} catch (BuggleInOuterSpaceException e) {
-			throw new RuntimeException("Broken lesson: you moved to outer space",e);
+			throw new RuntimeException("Broken lesson: you moved to outer space (at "+x+","+y+")",e);
 		}
 	}
 
@@ -275,7 +284,7 @@ public abstract class AbstractBuggle extends Entity {
 			return cell.hasLeftWall();
 
 		case Direction.SOUTH_VALUE: /* if looking down, look to the top of one cell lower */
-			cell = getCellFromLesson(getX(),                        (getY()+1) % getWorldHeight());
+			cell = getCellFromLesson(getX(), (getY()+1) % getWorldHeight());
 			return cell.hasTopWall();
 
 		case Direction.EAST_VALUE: /* if looking right, look to the left of one next cell */
@@ -295,7 +304,7 @@ public abstract class AbstractBuggle extends Entity {
 	private void move(Point delta) throws BuggleWallException {
 		if (delta == null)
 			return;
-		
+
 		int newx = (x + delta.x) % getWorldWidth();
 		if (newx < 0)
 			newx += getWorldWidth();
@@ -379,7 +388,7 @@ public abstract class AbstractBuggle extends Entity {
 	@Override
 	public String toString() {
 		return "Buggle (" + this.getClass().getName() + "): x=" + x + " y=" + y + " Direction:" + direction + " Color:"
-		+ bodyColor;
+				+ bodyColor;
 	}
 
 	@Override
@@ -395,6 +404,10 @@ public abstract class AbstractBuggle extends Entity {
 		return result;
 	}
 
+	public void ignoreDirectionDifference() {
+		dontIgnoreDirectionDifference = false;	
+	}
+	
 	@Override
 	public boolean equals(Object obj) {
 		if (this == obj)
@@ -403,17 +416,17 @@ public abstract class AbstractBuggle extends Entity {
 			return false;
 		if (!(obj instanceof AbstractBuggle))
 			return false;
-		
+
 		final AbstractBuggle other = (AbstractBuggle) obj;
 		if (bodyColor == null) {
 			if (other.bodyColor != null)
 				return false;
 		} else if (!bodyColor.equals(other.bodyColor))
 			return false;
-		if (direction == null) {
+		if (dontIgnoreDirectionDifference && direction == null) {
 			if (other.direction != null)
 				return false;
-		} else if (!direction.equals(other.direction))
+		} else if (dontIgnoreDirectionDifference && !direction.equals(other.direction))
 			return false;
 		if (seenError != other.seenError)
 			return false;
@@ -433,7 +446,7 @@ public abstract class AbstractBuggle extends Entity {
 		StringBuffer sb = new StringBuffer();
 		if (getX() != other.getX() || getY() != other.getY()) 
 			sb.append(Game.i18n.tr("    Its position is ({0},{1}); expected: ({2},{3}).\n",other.getX(),other.getY(),getX(),getY()));
-		if (getDirection() != other.getDirection()) 
+		if ((!dontIgnoreDirectionDifference) && getDirection() != other.getDirection()) 
 			sb.append(Game.i18n.tr("    Its direction is {0}; expected: {1}.\n",other.getDirection(),getDirection()));
 		if (getBodyColor() != other.getBodyColor()) 
 			sb.append(Game.i18n.tr("    Its color is {0}; expected: {1}.\n",other.getBodyColor(),getBodyColor()));
@@ -449,7 +462,7 @@ public abstract class AbstractBuggle extends Entity {
 			sb.append(Game.i18n.tr("    It didn't encounter any issue, such as bumping into a wall.\n"));
 		return sb.toString();
 	}
-	
+
 	/* BINDINGS TRANSLATION: French */
 	public void gauche()   { left(); }
 	public void droite()   { right(); }
@@ -482,5 +495,175 @@ public abstract class AbstractBuggle extends Entity {
 	// get/set X/Y/Pos are not translated as they happen to be the same in French
 	public boolean estChoisi()           { return isSelected(); } // we have to document the version without e, since po4a allows for one variant only
 	public boolean estChoisie()          { return isSelected(); } // But we want to have the grammatically correct form also possible (Buggles are feminine in French)
-	
+
+
+	@Override
+	public void command(String command, BufferedWriter out){
+		int num = Integer.parseInt((String) command.subSequence(0, 3));
+		int nb,nb2;
+		try {
+			switch(num){
+			case 110:
+				left();
+				break;
+			case 111:
+				right();
+				break;
+			case 112:
+				back();
+				break;
+			case 113 : 
+				nb = Integer.parseInt((command.split(" ")[1]));
+				if(nb==1){
+					forward();
+				}else{
+					forward(nb);
+				}
+				break;
+			case 114:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				if(nb==1){
+					backward();
+				}else{
+					backward(nb);
+				}
+				break;
+			case 115:
+				out.write(Integer.toString(getX()));
+				out.write("\n");
+				break;
+			case 116:
+				out.write(Integer.toString(getY()));
+				out.write("\n");
+				break;
+			case 117:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				setX(nb);
+				break;
+			case 118:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				setY(nb);
+				break;
+			case 119:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				nb2 = Integer.parseInt((command.split(" ")[2]));
+				setPos(nb, nb2);
+				break;
+			case 120:
+				out.write(Integer.toString(ColorMapper.color2int(getBodyColor())));
+				out.write("\n");
+				break;
+			case 121:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				setBodyColor(ColorMapper.int2color(nb));
+				break;
+			case 122:
+				out.write((isFacingWall()?"1":"0"));
+				out.write("\n");
+				break;
+			case 123:
+				out.write((isBackingWall()?"1":"0"));
+				out.write("\n");
+				break;	
+			case 124:
+				out.write(Integer.toString(getDirection().intValue()));
+				out.write("\n");
+				break;
+			case 125:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				Direction d=null;
+				switch(nb){
+				case Direction.NORTH_VALUE:
+					d=Direction.NORTH;
+					break;
+				case Direction.EAST_VALUE:
+					d=Direction.EAST;
+					break;
+				case Direction.SOUTH_VALUE:
+					d=Direction.SOUTH;
+					break;
+				case Direction.WEST_VALUE:
+					d=Direction.WEST;
+					break;
+				}
+				setDirection(d);
+				break;
+			case 126:
+				out.write((isSelected()?"1":"0"));
+				out.write("\n");
+				break;
+			case 127:
+				brushUp();
+				break;
+			case 128:
+				brushDown();
+				break;
+			case 129:
+				out.write((isBrushDown()?"1":"0"));
+				out.write("\n");
+				break;
+			case 130:
+				nb = Integer.parseInt((command.split(" ")[1]));
+				setBrushColor(ColorMapper.int2color(nb));
+				break;
+			case 131:
+				out.write(Integer.toString(ColorMapper.color2int(getBrushColor())));
+				out.write("\n");
+				System.out.println("a toi");
+				break;
+			case 132:
+				out.write(Integer.toString(ColorMapper.color2int(getGroundColor())));
+				out.write("\n");
+				break;
+			case 133:
+				out.write((isOverBaggle()?"1":"0"));
+				out.write("\n");
+				break;
+			case 134:
+				out.write((isCarryingBaggle()?"1":"0"));
+				out.write("\n");
+				break;
+			case 135:
+				pickupBaggle();
+				break;
+			case 136:
+				dropBaggle();
+				break;
+			case 137:
+				out.write((isOverMessage()?"1":"0"));
+				out.write("\n");
+				break;
+			case 138:
+				String mess = (command.split(" ")[1]);
+				writeMessage(mess);
+				break;
+			case 139:
+				out.write(readMessage());
+				out.write("\n");
+				break;
+			case 140:
+				clearMessage();
+				break;
+			case 141:
+				out.write(Integer.toString(getWorldHeight()));
+				out.write("\n");
+				break;
+			case 142:
+				out.write(Integer.toString(getWorldWidth()));
+				out.write("\n");
+				break;
+			default:
+				System.out.println("COMMANDE INCONNUE : "+command);
+				break;
+
+			}
+			out.flush();
+		} catch (IOException ioe) {
+			ioe.printStackTrace();
+		}catch (InvalidColorNameException ine) {
+			ine.printStackTrace();
+		}
+
+	}
+
 }
