@@ -12,8 +12,8 @@ import javax.script.ScriptEngine;
 import javax.script.ScriptException;
 import javax.swing.ImageIcon;
 
+import plm.core.lang.ProgrammingLanguage;
 import plm.core.model.Game;
-import plm.core.model.ProgrammingLanguage;
 import plm.core.ui.ResourcesCache;
 import plm.core.utils.ColorMapper;
 import plm.core.utils.FileUtils;
@@ -202,7 +202,7 @@ public class BuggleWorld extends GridWorld {
 				}
 				String buggleName = buggleMatcher.group(6);
 
-				new Buggle(res, buggleName, x, y, direction, color, brushColor);
+				new SimpleBuggle(res, buggleName, x, y, direction, color, brushColor);
 			} else if (cellMatcher.matches()) {
 				/* Get the info */
 				int x=Integer.parseInt( cellMatcher.group(1) );
@@ -346,6 +346,11 @@ public class BuggleWorld extends GridWorld {
 		result = PRIME * result + Arrays.hashCode(cells);
 		return result;
 	}
+	public World ignoreDirectionDifference() {
+		for (Entity e: getEntities()) 
+			((AbstractBuggle)e).ignoreDirectionDifference();
+		return this;
+	}
 
 	@Override
 	public boolean equals(Object obj) {
@@ -554,10 +559,14 @@ public class BuggleWorld extends GridWorld {
 			for (int y=0; y<getHeight(); y++) 
 				if (!getCell(x, y).equals(other.getCell(x, y))) 
 					sb.append(i18n.tr("  In ({0},{1})",x,y)+  getCell(x, y).diffTo(other.getCell(x, y))+".\n");
-		for (int i=0; i<entities.size(); i++)  
-			if (! entities.get(i).equals(other.entities.get(i))) 
-				sb.append(i18n.tr("  Something is wrong about buggle ''{0}'':\n",entities.get(i).getName())+
-						((AbstractBuggle) entities.get(i)).diffTo((AbstractBuggle) other.entities.get(i)));
+		if (entities.size() != other.entities.size()) {
+			sb.append(i18n.tr("  There is {0} entities where {1} were expected.",other.entities.size(),entities.size()));
+		} else {
+			for (int i=0; i<entities.size(); i++)  
+				if (! entities.get(i).equals(other.entities.get(i))) 
+					sb.append(i18n.tr("  Something is wrong about buggle ''{0}'':\n",entities.get(i).getName())+
+							((AbstractBuggle) entities.get(i)).diffTo((AbstractBuggle) other.entities.get(i)));
+		}
 		return sb.toString();
 	}
 
